import 'dart:convert';

class Location {
  String name;
  String? address;

  Location({required this.name, required this.address});

  factory Location.fromICalString(String icalString) {
    final name = _extractName(icalString);
    final address = _extractAddress(icalString);
    return Location(name: name, address: address);
  }

  static String _extractName(String icalString) {
    String name = icalString.split(';').first.split(':')[1];
    return name.trim();
  }

  static String? _extractAddress(String icalString) {
    if (icalString.contains('ADR=')) {
      String address = icalString.split(';').last.split(':')[1];
      return address.trim();
    }
    return null;
  }

  String toICalString() {
    final buffer = StringBuffer();
    buffer.write('LOCATION:$name');
    if (address != null) {
      buffer.write(';ADR=;$address');
    }
    return buffer.toString();
  }

  @override
  String toString() => '$name, $address';

  Map<String, dynamic> toMap() {
    return {
      'name': name,
      'address': address,
    };
  }

  factory Location.fromMap(Map<String, dynamic> map) {
    return Location(
      name: map['name'] ?? '',
      address: map['address'],
    );
  }

  String toJson() => json.encode(toMap());

  factory Location.fromJson(String source) =>
      Location.fromMap(json.decode(source));
}
