import 'dart:convert';

enum ICalStatus { tentative, confirmed, cancelled }

class EventStatus {
  final ICalStatus status;

  EventStatus({
    required this.status,
  });

  String toICalString() {
    switch (status) {
      case ICalStatus.tentative:
        return 'STATUS:TENTATIVE';
      case ICalStatus.confirmed:
        return 'STATUS:CONFIRMED';
      case ICalStatus.cancelled:
        return 'STATUS:CANCELLED';
    }
  }

  factory EventStatus.fromStatusString(String icalString) {
    switch (icalString) {
      case "TENTATIVE":
        return EventStatus(status: ICalStatus.tentative);
      case "CONFIRMED":
        return EventStatus(status: ICalStatus.confirmed);
      case "CANCELLED":
        return EventStatus(status: ICalStatus.cancelled);
      default:
        throw FormatException('Invalid iCalendar string: $icalString');
    }
  }

  EventStatus copyWith({
    ICalStatus? status,
  }) {
    return EventStatus(
      status: status ?? this.status,
    );
  }

  Map<String, dynamic> toMap() {
    return {'status': status.index};
  }

  factory EventStatus.fromMap(Map<String, dynamic> map) {
    return EventStatus(
      status: ICalStatus.values[map['status']],
    );
  }

  String toJson() => json.encode(toMap());

  factory EventStatus.fromJson(String source) =>
      EventStatus.fromMap(json.decode(source));

  @override
  String toString() => 'EventStatus(status: $status)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is EventStatus && other.status == status;
  }

  @override
  int get hashCode => status.hashCode;
}

class TodoStatus {
  final ICalStatus status;

  TodoStatus({required this.status});

  String toICalString() {
    switch (status) {
      case ICalStatus.tentative:
        return 'STATUS:TENTATIVE';
      case ICalStatus.confirmed:
        return 'STATUS:CONFIRMED';
      case ICalStatus.cancelled:
        return 'STATUS:CANCELLED';
    }
  }
}

class JournalStatus {
  final ICalStatus status;

  JournalStatus({required this.status});

  String toICalString() {
    switch (status) {
      case ICalStatus.tentative:
        return 'STATUS:TENTATIVE';
      case ICalStatus.confirmed:
        return 'STATUS:CONFIRMED';
      case ICalStatus.cancelled:
        return 'STATUS:CANCELLED';
    }
  }
}
